package com.unissoft.face;

import com.sun.jna.Pointer;
import com.unissoft.netSDKDemo.FMSGCallBack_V31;
import com.unissoft.netSDKDemo.HCNetSDK;
import lombok.extern.slf4j.Slf4j;

/**
 * 报警布防/监听功能模块：功能：报警布防/监听实时接受设备的人脸比对报警事件，回调函数中获取的报警类型(lCommand)为COMM_SNAP_MATCH_ALARM，报警信息(pAlarmInfo)对应结构体：NET_VCA_FACESNAP_MATCH_ALARM。
 * fSimilarity相似度大于0表示人脸比对成功，可以获取人脸库相关数据；fSimilarity相似度为0，需要通过NET_VCA_BLACKLIST_INFO结构体里面的byType类型判断，值为1表示陌生人模式，值为2表示人脸比对失败。
 */
@Slf4j
public class Alarm {

    public static HCNetSDK.FMSGCallBack_V31 fMSFCallBack_V31 = null;

    /**
     * 报警布防 （布防和监听选其一）
     *
     * @param lUserID 用户登录句柄
     */
    public static void SetAlarm(int lUserID) {
        //尚未布防,需要布防
        if (FaceMain.lAlarmHandle < 0) {

            //设置报警回调函数，一个程序进程只需要而且只能调用一次，对接多台设备，可以SDK初始化之后就调用该接口设置报警回调函数
            if (fMSFCallBack_V31 == null) {
                fMSFCallBack_V31 = new FMSGCallBack_V31();
                Pointer pUser = null;
                if (!FaceMain.hCNetSDK.NET_DVR_SetDVRMessageCallBack_V31(fMSFCallBack_V31, pUser)) {
                    log.error("设置回调函数失败!");
                } else {
                    log.info("设置回调函数成功!");
                }
            }

            //报警布防参数设置
            HCNetSDK.NET_DVR_SETUPALARM_PARAM m_strAlarmInfo = new HCNetSDK.NET_DVR_SETUPALARM_PARAM();
            m_strAlarmInfo.dwSize = m_strAlarmInfo.size();
            m_strAlarmInfo.byLevel = 1;  //布防等级
            m_strAlarmInfo.byAlarmInfoType = 1;   // 智能交通报警信息上传类型：0- 老报警信息（NET_DVR_PLATE_RESULT），1- 新报警信息(NET_ITS_PLATE_RESULT)
            m_strAlarmInfo.byDeployType = 1;   //布防类型 0：客户端布防 1：实时布防
            m_strAlarmInfo.write();
            FaceMain.lAlarmHandle = FaceMain.hCNetSDK.NET_DVR_SetupAlarmChan_V41(lUserID, m_strAlarmInfo);
            log.info("lAlarmHandle: {}", FaceMain.lAlarmHandle);
            if (FaceMain.lAlarmHandle == -1) {
                log.error("布防失败，错误码为: {}", FaceMain.hCNetSDK.NET_DVR_GetLastError());
            } else {
                log.info("布防成功");
            }
        }
    }

    /**
     * 报警监听
     * <p>
     * 注意: 设置监听前需要现在本机pc配置端口
     * 具体步骤：库文件ClientDemo->登录设备->配置->网络参数配置->配置告警管理主机地址、告警管理主机端口（需要与NET_DVR_StartListen_V30接口中对应）
     */
    public static void StartListen() {
        if (fMSFCallBack_V31 == null) {
            fMSFCallBack_V31 = new FMSGCallBack_V31();
            Pointer pUser = null;
            if (!FaceMain.hCNetSDK.NET_DVR_SetDVRMessageCallBack_V31(fMSFCallBack_V31, pUser)) {
                log.error("设置回调函数失败!");
            } else {
                log.info("设置回调函数成功!");
            }
        }
        /**
         * 这里NET_DVR_StartListen_V30接口中的ip和端口需要和配置的ip和端口对应
         * NET_DVR_StartListen_V30中传入的IP和端口是服务器或者测试PC的IP地址和端口，不是设备的IP和端口！！！
         */
        FaceMain.lListenHandle = FaceMain.hCNetSDK.NET_DVR_StartListen_V30(null, (short) 17201, fMSFCallBack_V31, null);
        if (FaceMain.lListenHandle == -1) {
            log.error("在端口17201启动报警监听失败: {}", FaceMain.hCNetSDK.NET_DVR_GetLastError());
        } else {
            log.info("在端口17201启动报警监听成功");
        }
    }

}
