package com.unissoft.face;

import com.sun.jna.Native;
import com.sun.jna.Pointer;
import com.unissoft.common.osSelect;
import com.unissoft.netSDKDemo.HCNetSDK;
import com.unissoft.pushMsg.AddressModel;
import lombok.extern.slf4j.Slf4j;
import org.dom4j.util.StringUtils;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.time.LocalDate;
import java.util.Calendar;
import java.util.Date;
import java.util.Scanner;

@Slf4j
@Component
@Order(1)
public class FaceMain implements ApplicationRunner {

    static HCNetSDK hCNetSDK = null;
    static int lUserID = -1;//用户句柄
    static int lAlarmHandle = -1; //布防句柄
    static int lListenHandle = -1; //监听句柄

    // 设备连接信息
    @Resource
    private AddressModel addressModel;

    /*
     * 获取配置
     */
    private void mainMethod() {
        log.info("FaceMain mainMethod===========================================================init thread");

        try {
            if (hCNetSDK == null) {
                if (!CreateSDKInstance()) {
                    log.error("Load SDK fail");
                    return;
                }
            }

            //linux系统建议调用以下接口加载组件库
            if (osSelect.isLinux()) {
                System.out.println("linux system come in here");
                HCNetSDK.BYTE_ARRAY ptrByteArray1 = new HCNetSDK.BYTE_ARRAY(256);
                HCNetSDK.BYTE_ARRAY ptrByteArray2 = new HCNetSDK.BYTE_ARRAY(256);
                //这里是库的绝对路径，请根据实际情况修改，注意改路径必须有访问权限
                String strPath1 = System.getProperty("user.dir") + "/lib/libcrypto.so.1.1";
                String strPath2 = System.getProperty("user.dir") + "/lib/libssl.so.1.1";
                System.out.println("strPath1:" + strPath1);

                System.arraycopy(strPath1.getBytes(), 0, ptrByteArray1.byValue, 0, strPath1.length());
                ptrByteArray1.write();
                hCNetSDK.NET_DVR_SetSDKInitCfg(3, ptrByteArray1.getPointer());

                System.arraycopy(strPath2.getBytes(), 0, ptrByteArray2.byValue, 0, strPath2.length());
                ptrByteArray2.write();
                hCNetSDK.NET_DVR_SetSDKInitCfg(4, ptrByteArray2.getPointer());

                String strPathCom = System.getProperty("user.dir") + "/lib";
                HCNetSDK.NET_DVR_LOCAL_SDK_PATH struComPath = new HCNetSDK.NET_DVR_LOCAL_SDK_PATH();
                System.arraycopy(strPathCom.getBytes(), 0, struComPath.sPath, 0, strPathCom.length());
                struComPath.write();
                hCNetSDK.NET_DVR_SetSDKInitCfg(2, struComPath.getPointer());
            }
            //初始化
            hCNetSDK.NET_DVR_Init();

            //加载日志
            hCNetSDK.NET_DVR_SetLogToFile(3, "..//sdklog", false);

            //登录设备，每一台设备分别登录; 登录句柄是唯一的，可以区分设备
            HCNetSDK.NET_DVR_USER_LOGIN_INFO m_strLoginInfo = new HCNetSDK.NET_DVR_USER_LOGIN_INFO();//设备登录信息
            HCNetSDK.NET_DVR_DEVICEINFO_V40 m_strDeviceInfo = new HCNetSDK.NET_DVR_DEVICEINFO_V40();//设备信息
            //m_strDeviceInfo.struDeviceV30.
            String m_sDeviceIP = addressModel.getDeviceIP();//设备ip地址
            m_strLoginInfo.sDeviceAddress = new byte[HCNetSDK.NET_DVR_DEV_ADDRESS_MAX_LEN];
            System.arraycopy(m_sDeviceIP.getBytes(), 0, m_strLoginInfo.sDeviceAddress, 0, m_sDeviceIP.length());

            String m_sUsername = addressModel.getUserName();//设备用户名
            m_strLoginInfo.sUserName = new byte[HCNetSDK.NET_DVR_LOGIN_USERNAME_MAX_LEN];
            System.arraycopy(m_sUsername.getBytes(), 0, m_strLoginInfo.sUserName, 0, m_sUsername.length());

            String m_sPassword = addressModel.getPassword();//设备密码
            m_strLoginInfo.sPassword = new byte[HCNetSDK.NET_DVR_LOGIN_PASSWD_MAX_LEN];
            System.arraycopy(m_sPassword.getBytes(), 0, m_strLoginInfo.sPassword, 0, m_sPassword.length());

            m_strLoginInfo.wPort = 8000;
            m_strLoginInfo.bUseAsynLogin = false; //是否异步登录：0- 否，1- 是
            m_strLoginInfo.byLoginMode = 0;  //登录模式：0- SDK私有协议，1- ISAPI协议
            m_strLoginInfo.write();


            lUserID = hCNetSDK.NET_DVR_Login_V40(m_strLoginInfo, m_strDeviceInfo);
            if (lUserID == -1) {
                log.error("登录失败,错误码为: {}", hCNetSDK.NET_DVR_GetLastError());
            } else {
                log.info("{} 设备登录成功!设备序列号: {}", m_sDeviceIP, new String(m_strDeviceInfo.struDeviceV30.sSerialNumber).trim());
                m_strDeviceInfo.read();
            }

            try {
                log.info("進行撤防");
                hCNetSDK.NET_DVR_CloseAlarmChan_V30(lAlarmHandle);
            } catch (Exception e) {
                log.info("不需要撤防");
            }
            try {
                log.info("停止监听");
                hCNetSDK.NET_DVR_StopListen_V30(lListenHandle);
            } catch (Exception e) {
                log.info("不需要停止监听");
            }

            //报警布防实时接收人脸比对的结果信息，也可以单独参考“2-报警布防监听”这个Demo示例
            Alarm.SetAlarm(lUserID);

            // TODO 暂停状态 需要测试改造
            Date satart = new Date();
            Calendar c1 = Calendar.getInstance();
            c1.setTime(satart);
            while (true) {
                //这里加入控制台输入控制，是为了保持连接状态，当输入Y表示布防结束
                log.info("监听进行中，输入字母Q可退出监听");
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(System.in));
                String str = bufferedReader.readLine();
                if (str.toUpperCase().equals("Q")) {
                    break;
                 /*
                Thread.sleep(60000);
                Date end = new Date();
                Calendar c2 = Calendar.getInstance();
                c2.setTime(end);
                long millis = c2.getTimeInMillis() - c1.getTimeInMillis();
                long hours = millis/(1000*60*60);
                if(hours > 10) {
                    log.info("时间过去10小时，退出程序");
                    break;
                }
                }*/
                }

                //退出之前判断布防监听状态，并做撤防和停止监听操作
                if (lAlarmHandle >= 0) {
                    if (!hCNetSDK.NET_DVR_CloseAlarmChan_V30(lAlarmHandle)) {
                        log.error("撤防失败，错误码: {}", hCNetSDK.NET_DVR_GetLastError());
                    } else {
                        log.info("撤防成功！！！");
                    }
                }
                if (lListenHandle >= 0) {
                    if (!hCNetSDK.NET_DVR_StopListen_V30(lListenHandle)) {
                        log.error("取消监听失败，错误码：{}", hCNetSDK.NET_DVR_GetLastError());
                    } else {
                        log.info("停止监听成功！！！");
                    }
                }

                //注销登录，程序退出时调用；程序同时对接多台设备时，每一台设备分别调用一次接口，传入不同的登录句柄
                if (lUserID >= 0) {
                    hCNetSDK.NET_DVR_Logout(lUserID);
                }

                //SDK反初始化，释放资源，程序退出时调用
                boolean b = hCNetSDK.NET_DVR_Cleanup();
                if (b)
                    log.info("释放资源,程序退出!");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据不同操作系统选择不同的库文件和库路径
     *
     * @return
     */
    private boolean CreateSDKInstance() {
        if (hCNetSDK == null) {
            synchronized (HCNetSDK.class) {
                String strDllPath = "";
                try {
//                    System.setProperty("jna.debug_load", "true");
                    if (osSelect.isWindows())
                        //win系统加载库路径
                        strDllPath = System.getProperty("user.dir") + "\\lib\\HCNetSDK.dll";
                    else if (osSelect.isLinux())
                        //Linux系统加载库路径
                        strDllPath = System.getProperty("user.dir") + "/lib/libhcnetsdk.so";
                    hCNetSDK = (HCNetSDK) Native.loadLibrary(strDllPath, HCNetSDK.class);
                } catch (Exception ex) {
                    log.error("loadLibrary: {} Error: {}", strDllPath, ex.getMessage());
                    return false;
                }
            }
        }
        return true;
    }

    private void alarmListenMethod() {

        if (hCNetSDK == null) {
            if (!CreateSDKInstance()) {
                log.error("Load SDK fail");
                return;
            }
        }

        //linux系统建议调用以下接口加载组件库
        if (osSelect.isLinux()) {
            System.out.println("linux system come in here");
            HCNetSDK.BYTE_ARRAY ptrByteArray1 = new HCNetSDK.BYTE_ARRAY(256);
            HCNetSDK.BYTE_ARRAY ptrByteArray2 = new HCNetSDK.BYTE_ARRAY(256);
            //这里是库的绝对路径，请根据实际情况修改，注意改路径必须有访问权限
            String strPath1 = System.getProperty("user.dir") + "/lib/libcrypto.so.1.1";
            String strPath2 = System.getProperty("user.dir") + "/lib/libssl.so.1.1";
            System.out.println("strPath1:" + strPath1);

            System.arraycopy(strPath1.getBytes(), 0, ptrByteArray1.byValue, 0, strPath1.length());
            ptrByteArray1.write();
            hCNetSDK.NET_DVR_SetSDKInitCfg(3, ptrByteArray1.getPointer());

            System.arraycopy(strPath2.getBytes(), 0, ptrByteArray2.byValue, 0, strPath2.length());
            ptrByteArray2.write();
            hCNetSDK.NET_DVR_SetSDKInitCfg(4, ptrByteArray2.getPointer());

            String strPathCom = System.getProperty("user.dir") + "/lib";
            HCNetSDK.NET_DVR_LOCAL_SDK_PATH struComPath = new HCNetSDK.NET_DVR_LOCAL_SDK_PATH();
            System.arraycopy(strPathCom.getBytes(), 0, struComPath.sPath, 0, strPathCom.length());
            struComPath.write();
            hCNetSDK.NET_DVR_SetSDKInitCfg(2, struComPath.getPointer());
        }
        //初始化
        hCNetSDK.NET_DVR_Init();
        Alarm.StartListen();
        try {
            while (true) {
                Thread.sleep(60000);
                log.info("监听进行中，如需退出监听请输入字母Q");
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(System.in));
                String str = bufferedReader.readLine();
                if (str.toUpperCase().equals("Q")) {
                    break;
                }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }

        //停止监听
        if (lListenHandle >= 0) {
            if (!hCNetSDK.NET_DVR_StopListen_V30(lListenHandle)) {
                log.error("取消监听失败，错误码：{}", hCNetSDK.NET_DVR_GetLastError());
            } else {
                log.info("停止监听成功！！！");
            }
        }
        hCNetSDK.NET_DVR_Cleanup();
    }
    @Override
    public void run(ApplicationArguments args) throws Exception {
        //after behind for application run,because implements ApplicationRunner
        this.mainMethod();
        //this.alarmListenMethod();
    }
}
