package com.unissoft.netSDKDemo;

import com.alibaba.fastjson.JSON;
import com.sun.jna.Pointer;
import com.unissoft.pushMsg.AlarmDSService;
import com.unissoft.pushMsg.AlarmDSServiceImpl;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 人脸比对报警接收回调函数
 */
@Slf4j
public class FMSGCallBack_V31 implements HCNetSDK.FMSGCallBack_V31 {
    // this class call jna native callback,so is not spring manage class
    private AlarmDSService alarmDSService = new AlarmDSServiceImpl();

    //报警信息回调函数
    public boolean invoke(int lCommand, HCNetSDK.NET_DVR_ALARMER pAlarmer, Pointer pAlarmInfo, int dwBufLen, Pointer pUser) {
        AlarmDataHandle(lCommand, pAlarmer, pAlarmInfo, dwBufLen, pUser);
        return true;
    }

    public void AlarmDataHandle(int lCommand, HCNetSDK.NET_DVR_ALARMER pAlarmer, Pointer pAlarmInfo, int dwBufLen, Pointer pUser) {
        log.info("报警事件发生,进入回调");
        /*String hexString = Integer.toHexString(lCommand);
        log.info("报警类型 hexString {}", hexString);*/
        HCNetSDK.NET_VCA_RULE_ALARM ruleAlarm;
        Pointer ruleAlarmPointer;
        switch (lCommand) {
            case HCNetSDK.COMM_ALARM_RULE://行为分析信息上传

                ruleAlarm = new HCNetSDK.NET_VCA_RULE_ALARM();
                ruleAlarm.write();
                ruleAlarmPointer = ruleAlarm.getPointer();
                ruleAlarmPointer.write(0, pAlarmInfo.getByteArray(0, ruleAlarm.size()), 0, ruleAlarm.size());
                ruleAlarm.read();
                //log.info("alarm type in the COMM_ALARM_RULE ---------->: {}", JSON.toJSONString(ruleAlarm));
                System.out.println("行为类型：" + ruleAlarm.struRuleInfo.wEventTypeEx);
                System.out.println("报警通道对应设备的通道号：" + ruleAlarm.struDevInfo.byChannel);
                System.out.println("SDK接入设备的通道号：" + ruleAlarm.struDevInfo.byIvmsChannel);
                System.out.println("报警通道对应设备的IP地址：" + ruleAlarm.struDevInfo.struDevIP.toString());
                System.out.println("报警通道对应设备的端口号：" + ruleAlarm.struDevInfo.wPort);

                
                alarmDSService.pushLCommandEx(lCommand, ruleAlarm);
                log.info(
                        "行为分析事件，lCommand {} -> pAlarmer {} -> pAlarmInfo {} -> dwBufLen {} -> pUser {}",
                        lCommand, JSON.toJSONString(pAlarmer), JSON.toJSONString(pAlarmInfo),dwBufLen,JSON.toJSONString(pUser)
                );
                break;

            case HCNetSDK.COMM_VEHICLE_CONTROL_ALARM://车辆报警上传
                log.info(
                        "车辆报警事件，lCommand {} -> pAlarmer {} -> pAlarmInfo {} -> dwBufLen {} -> pUser {}",
                        lCommand, JSON.toJSONString(pAlarmer), JSON.toJSONString(pAlarmInfo),dwBufLen,JSON.toJSONString(pUser)
                );
                ruleAlarm = new HCNetSDK.NET_VCA_RULE_ALARM();
                ruleAlarm.write();
                ruleAlarmPointer = ruleAlarm.getPointer();
                ruleAlarmPointer.write(0, pAlarmInfo.getByteArray(0, ruleAlarm.size()), 0, ruleAlarm.size());
                ruleAlarm.read();
                //log.info("alarm type in the COMM_VEHICLE_CONTROL_ALARM ---------->: {}", JSON.toJSONString(ruleAlarm));
                alarmDSService.pushLCommand(lCommand);
                break;
            case HCNetSDK.COMM_ALARMHOST_CID_ALARM:
                log.info("报警主机防区报警");
                break;
            case HCNetSDK.COMM_SNAP_MATCH_ALARM:
                //人脸黑名单比对报警
                log.info("SNAP_MATCH_ALARM");
                HCNetSDK.NET_VCA_FACESNAP_MATCH_ALARM strFaceSnapMatch = new HCNetSDK.NET_VCA_FACESNAP_MATCH_ALARM();
                strFaceSnapMatch.write();
                Pointer pFaceSnapMatch = strFaceSnapMatch.getPointer();
                pFaceSnapMatch.write(0, pAlarmInfo.getByteArray(0, strFaceSnapMatch.size()), 0, strFaceSnapMatch.size());
                strFaceSnapMatch.read();
                //比对结果
                log.info("比对结果: {}", strFaceSnapMatch.byContrastStatus == 1 ? "比对成功" : "比对失败");
                //人脸库ID
                if (strFaceSnapMatch.struBlockListInfo.dwFDIDLen > 0) {
                    long offset1 = 0;
                    ByteBuffer buffers1 = strFaceSnapMatch.struBlockListInfo.pFDID.getByteBuffer(offset1, strFaceSnapMatch.struBlockListInfo.dwFDIDLen);
                    byte[] bytes1 = new byte[strFaceSnapMatch.struBlockListInfo.dwFDIDLen];
                    buffers1.get(bytes1);
                    log.info("人脸库ID: {}", new String(bytes1));
                }
                //人脸库图片ID
                if (strFaceSnapMatch.struBlockListInfo.dwPIDLen > 0) {
                    long offset2 = 0;
                    ByteBuffer buffers2 = strFaceSnapMatch.struBlockListInfo.pPID.getByteBuffer(offset2, strFaceSnapMatch.struBlockListInfo.dwPIDLen);
                    byte[] bytes2 = new byte[strFaceSnapMatch.struBlockListInfo.dwPIDLen];
                    buffers2.get(bytes2);
                    log.info("图片ID: {}", new String(bytes2));
                }
                //保存抓拍人脸图，图片数据传输方式: 0- 二进制
                if ((strFaceSnapMatch.dwSnapPicLen > 0) && (strFaceSnapMatch.byPicTransType == 0)) {
                    SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss");
                    String newName = sf.format(new Date());
                    FileOutputStream fout;
                    try {
                        String filename = ".//pic//" + newName + "_pSnapPicBuffer" + ".jpg";
                        fout = new FileOutputStream(filename);
                        //将字节写入文件
                        long offset = 0;
                        ByteBuffer buffers = strFaceSnapMatch.pSnapPicBuffer.getByteBuffer(offset, strFaceSnapMatch.dwSnapPicLen);
                        byte[] bytes = new byte[strFaceSnapMatch.dwSnapPicLen];
                        buffers.rewind();
                        buffers.get(bytes);
                        fout.write(bytes);
                        fout.close();
                    } catch (IOException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
                //保存抓拍人脸图URL，图片数据传输方式: 1- URL 注：超脑设备人脸比对报警图片一般按照URL上传，根据URL去下载设备中存储的图片，认证方式选择：摘要认证
                if ((strFaceSnapMatch.dwSnapPicLen > 0) && (strFaceSnapMatch.byPicTransType == 1)) {
                    long offset = 0;
                    ByteBuffer buffers = strFaceSnapMatch.pSnapPicBuffer.getByteBuffer(offset, strFaceSnapMatch.dwSnapPicLen);
                    byte[] bytes = new byte[strFaceSnapMatch.dwSnapPicLen];
                    buffers.rewind();
                    buffers.get(bytes);
                    log.info("人脸抓拍图片 URL: {}", new String(bytes));
                }
                if ((strFaceSnapMatch.struSnapInfo.dwSnapFacePicLen > 0) && (strFaceSnapMatch.byPicTransType == 0)) {
                    SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss");
                    String newName = sf.format(new Date());
                    FileOutputStream fout;
                    try {
                        String filename = ".//pic//" + newName + "_struSnapInfo_pBuffer1" + ".jpg";
                        fout = new FileOutputStream(filename);
                        //将字节写入文件
                        long offset = 0;
                        ByteBuffer buffers = strFaceSnapMatch.struSnapInfo.pBuffer1.getByteBuffer(offset, strFaceSnapMatch.struSnapInfo.dwSnapFacePicLen);
                        byte[] bytes = new byte[strFaceSnapMatch.struSnapInfo.dwSnapFacePicLen];
                        buffers.rewind();
                        buffers.get(bytes);
                        fout.write(bytes);
                        fout.close();
                    } catch (IOException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
                if ((strFaceSnapMatch.struSnapInfo.dwSnapFacePicLen > 0) && (strFaceSnapMatch.byPicTransType == 1)) {
                    long offset = 0;
                    ByteBuffer buffers = strFaceSnapMatch.struSnapInfo.pBuffer1.getByteBuffer(offset, strFaceSnapMatch.struSnapInfo.dwSnapFacePicLen);
                    byte[] bytes = new byte[strFaceSnapMatch.struSnapInfo.dwSnapFacePicLen];
                    buffers.rewind();
                    buffers.get(bytes);
                    log.info("人脸抓拍图片1 URL: {}", new String(bytes));
                }
                //保存人脸库图片，图片数据传输方式: 0- 二进制
                if ((strFaceSnapMatch.struBlockListInfo.dwBlockListPicLen > 0) && (strFaceSnapMatch.byPicTransType == 0)) {
                    SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss");
                    String newName = sf.format(new Date());
                    FileOutputStream fout;
                    try {
                        String filename = ".//pic//" + newName + "_fSimilarity_" + strFaceSnapMatch.fSimilarity + "_struBlackListInfo_pBuffer1" + ".jpg";
                        fout = new FileOutputStream(filename);
                        //将字节写入文件
                        long offset = 0;
                        ByteBuffer buffers = strFaceSnapMatch.struBlockListInfo.pBuffer1.getByteBuffer(offset, strFaceSnapMatch.struBlockListInfo.dwBlockListPicLen);
                        byte[] bytes = new byte[strFaceSnapMatch.struBlockListInfo.dwBlockListPicLen];
                        buffers.rewind();
                        buffers.get(bytes);
                        fout.write(bytes);
                        fout.close();

                    } catch (IOException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
                //保存人脸库图片，图片数据传输方式: 1-URL
                if ((strFaceSnapMatch.struBlockListInfo.dwBlockListPicLen > 0) && (strFaceSnapMatch.byPicTransType == 1)) {
                    long offset = 0;
                    ByteBuffer buffers = strFaceSnapMatch.struBlockListInfo.pBuffer1.getByteBuffer(offset, strFaceSnapMatch.struBlockListInfo.dwBlockListPicLen);
                    byte[] bytes = new byte[strFaceSnapMatch.struBlockListInfo.dwBlockListPicLen];
                    buffers.rewind();
                    buffers.get(bytes);
                    log.info("人脸库图片 URL: {}", new String(bytes));
                }
                //抓拍库附加信息数据指针，附加信息的XML文本中包含人脸温度数据
                if (strFaceSnapMatch.struBlockListInfo.struBlockListInfo.dwFCAdditionInfoLen > 0) {
                    long offset = 0;
                    ByteBuffer buffers = strFaceSnapMatch.struBlockListInfo.struBlockListInfo.pFCAdditionInfoBuffer.getByteBuffer(offset, strFaceSnapMatch.struBlockListInfo.struBlockListInfo.dwFCAdditionInfoLen);
                    byte[] bytes = new byte[strFaceSnapMatch.struBlockListInfo.struBlockListInfo.dwFCAdditionInfoLen];
                    buffers.rewind();
                    buffers.get(bytes);
                    log.info("抓拍库附加信息: {}", new String(bytes));
                }
                alarmDSService.pushLCommand(lCommand);
                break;
            default:
                // TODO 单报警设备 目前不需要 sAlarmType alarmNow
                //log.info("alarm type in the FMSGCallBack_V31 ->int: {}", lCommand);
                log.info(
                        "不分类事件，不返回报警具体数据，lCommand {} -> pAlarmer {} -> pAlarmInfo {} -> dwBufLen {} -> pUser {}",
                        lCommand, JSON.toJSONString(pAlarmer), JSON.toJSONString(pAlarmInfo),dwBufLen,JSON.toJSONString(pUser)
                );
                alarmDSService.pushLCommand(lCommand);
                break;
        }
    }

}
