package com.unissoft.pushMsg;

import com.unissoft.netSDKDemo.HCNetSDK;
import com.unissoft.utils.GetBeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author qingsong.han
 * @description: 推送 异常闯入/入侵
 * @create 2022-10-14 11:03
 */
@Slf4j
@Service
public class AlarmDSServiceImpl implements AlarmDSService {

    @Override
    public void pushLCommand(Integer lCommand) {
        // 获取spring RestTemple bean
        RestTemplate restTemplate = (RestTemplate) GetBeanUtil.getBean("restTemplate");
        // get push class AddressModel bean
        AddressModel addressModel = (AddressModel) GetBeanUtil.getBean("addressModel");
        // MultiValueMap requestEntity
        MultiValueMap<String, Object> requestEntity = new LinkedMultiValueMap<>();
        // 报警类型(异常闯入/入侵) int 4354
        requestEntity.add("sAlarmType", lCommand);
        // 报警时间
        requestEntity.add("alarmNow", new Date());
        // 设备ip
        requestEntity.add("deviceIP", addressModel.getDeviceIP());
        // 机构编码
        requestEntity.add("orgCode", addressModel.getOrgCode());
        log.info("报警类型 int: {}", requestEntity);
        // push of post for object
        Object object = restTemplate.postForObject(addressModel.getPush(), requestEntity, Object.class);
        log.info("AlarmDSServiceImpl push alarm result: {}", object);
    }

    @Override
    public void pushLCommandEx(Integer lCommand, HCNetSDK.NET_VCA_RULE_ALARM ruleAlarm) {
        // 获取spring RestTemple bean
        RestTemplate restTemplate = (RestTemplate) GetBeanUtil.getBean("restTemplate");
        // get push class AddressModel bean
        AddressModel addressModel = (AddressModel) GetBeanUtil.getBean("addressModel");
        // MultiValueMap requestEntity
        MultiValueMap<String, Object> requestEntity = new LinkedMultiValueMap<>();
        // 报警类型(异常闯入/入侵) int 4354
        requestEntity.add("sAlarmType", lCommand);
        // 报警时间
        requestEntity.add("alarmNow", new Date());
        // 设备ip
        requestEntity.add("deviceIP", addressModel.getDeviceIP());
        // 机构编码
        requestEntity.add("orgCode", addressModel.getOrgCode());

        requestEntity.add("cameraIP", ruleAlarm.struDevInfo.struDevIP.toString());
        //将字节写入文件
        SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss");
        String newName = sf.format(new Date());
        FileOutputStream fout;
        String filename = ".//pic//" + newName + "_pSnapPicBuffer" + ".jpg";
        try {
            long offset = 0;
            fout = new FileOutputStream(filename);
            ByteBuffer buffers = ruleAlarm.pImage.getByteBuffer(offset, ruleAlarm.dwPicDataLen);
            byte[] bytes = new byte[ruleAlarm.dwPicDataLen];
            buffers.rewind();
            buffers.get(bytes);
            fout.write(bytes);
            fout.close();
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        long offset = 0;
        ByteBuffer buffers = ruleAlarm.pImage.getByteBuffer(offset, ruleAlarm.dwPicDataLen);
        byte[] bytes = new byte[ruleAlarm.dwPicDataLen];
        buffers.rewind();
        buffers.get(bytes);
        buffers.flip();
        String picStr = StandardCharsets.UTF_8.decode(buffers).toString();
        log.info("pic-string length:"+picStr.length());
        requestEntity.add("pic", picStr);
        requestEntity.add("picSize", ruleAlarm.dwPicDataLen);
        log.info("报警类型: %d", requestEntity.get("sAlarmType") + "超脑IP："+requestEntity.get("deviceIP")+"\t摄像头IP："+requestEntity.get("cameraIP"));
        // push of post for object
        //Object object = restTemplate.postForObject(addressModel.getPush(), requestEntity, Object.class);
        //log.info("AlarmDSServiceImpl push alarm result: {}", object);

        RestTemplate rest = new RestTemplate();
        MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
        headers.add("content-type", MediaType.APPLICATION_FORM_URLENCODED_VALUE);
        // headers.add("Accept", MediaType.APPLICATION_FORM_URLENCODED_VALUE);
        // 请求体
        MultiValueMap<String, String> body = new LinkedMultiValueMap<>();
        body.add("file", "picture");
        body.add("base64", Base64.encodeBase64String(bytes));
        body.add("key", "123456");

        HttpEntity<MultiValueMap<String, String>> httpEntity = new HttpEntity<>(body, headers);
        ResponseEntity<String> response = rest.postForEntity(addressModel.getPush(), httpEntity, String.class);
    }
}
